/*
  Simple Demo for GLSL 2.0

  Modified from www.lighthouse3d.com

  Francois Faure, 2014
*/
#include <stdio.h>
#include <stdlib.h>

#include <GL/glew.h>
#include <GL/glut.h>
#include "helper.h"
#include "traqueboule.h"


float lpos[4] = {-1,0.5,2,1};


void changeSize(int w, int h) {
	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;

	float ratio = 1.0* w / h;

	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	// Set the viewport to be the entire window
    glViewport(0, 0, w, h);

	// Set the correct perspective.
	gluPerspective(45,ratio,1,1000);
	glMatrixMode(GL_MODELVIEW);
}


void renderScene(void) {

    glEnable(GL_DEPTH_TEST);
    glClearColor(0.0,0.0,0.0,1.0);

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
    tbVisuTransform();

    glLightfv(GL_LIGHT0, GL_POSITION, lpos);

//    glutSolidTeapot(1);
    glutSolidSphere(1,16,16);

	glutSwapBuffers();
}

void processNormalKeys(unsigned char key, int /*x*/, int /*y*/) {

	if (key == 27) 
		exit(0);
}


void setShaders() {


    GLuint v = glCreateShader(GL_VERTEX_SHADER);
    GLuint f = glCreateShader(GL_FRAGMENT_SHADER);
    GLuint f2 = glCreateShader(GL_FRAGMENT_SHADER);


    char *vs = textFileRead("gouraud.vert");
    char *fs = textFileRead("gouraud.frag");
//    char *vs = textFileRead("phong.vert");
//    char *fs = textFileRead("phong.frag");
//    char *vs = textFileRead("toon.vert");
//    char *fs = textFileRead("toon.frag");
    char *fs2 = textFileRead("toon2.frag");

	const char * ff = fs;
	const char * ff2 = fs2;
	const char * vv = vs;

	glShaderSource(v, 1, &vv,NULL);
    glShaderSource(f, 1, &ff,NULL);
	glShaderSource(f2, 1, &ff2,NULL);

	free(vs);free(fs);

    glCompileShader(v); printShaderInfoLog(v);
    glCompileShader(f); printShaderInfoLog(f);
//	glCompileShader(f2);

    GLuint p = glCreateProgram();
    glAttachShader(p,f);
//    glAttachShader(p,f2);
    glAttachShader(p,v);

    glLinkProgram(p); printProgramInfoLog(p);
    glUseProgram(p);
    printf("shader program created\n");
    fflush(stdout);
}


int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(320,320);
    glutCreateWindow("Simple glut-glsl application");

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);
	glutReshapeFunc(changeSize);
	glutKeyboardFunc(processNormalKeys);

    // interactive view
    glutMouseFunc(tbMouseFunc);    // traqueboule gets the mouse
    glutMotionFunc(tbMotionFunc);  // traqueboule gets the mouse
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(0.0,0.0,5.0,  0.0,0.0,-1.0, 0.0f,1.0f,0.0f);
    tbInitTransform();     // set current transform as default view

    // shaders
	glewInit();
	if (glewIsSupported("GL_VERSION_2_0"))
		printf("Ready for OpenGL 2.0\n");
	else {
		printf("OpenGL 2.0 not supported\n");
		exit(1);
	}
    setShaders();

	glutMainLoop();

	// just for compatibiliy purposes
	return 0;
}

