/** \file traqueboule.h

Utilitaires pour appliquer une traqueboule a une fenetre OpenGL.

*/
#ifndef TRAQUEBOULE
#define TRAQUEBOULE
#include <GL/glut.h>
#include <math.h>

/** votre fonction d'affichage */
void display();

/** Placement de la scene par rapport a la camera */
GLfloat tb_translation[3] = { 0,0,-3.6 },
        tb_rotation[16] = { 1,0,0,0,
	                        0,1,0,0,
							0,0,1,0,
							0,0,0,1  };
/** Gestion de la souris */
int tb_ancienX, tb_ancienY, tb_tournerXY=0, tb_translaterXY=0, tb_bougerZ=0;


/** Lit dans la matrice courante la position initiale du point de vue */
void tbInitTransform()
{
	glGetFloatv( GL_MODELVIEW_MATRIX, tb_rotation );
	tb_translation[0] = tb_rotation[12]; tb_rotation[12] = 0;
	tb_translation[1] = tb_rotation[13]; tb_rotation[13] = 0;
	tb_translation[2] = tb_rotation[14]; tb_rotation[14] = 0;
}

/** Applique la transformation de point de vue */
void tbVisuTransform()
{
  glTranslatef( tb_translation[0], tb_translation[1], tb_translation[2] );
  glMultMatrixf( tb_rotation );
};

/** Affiche l'aide */
void tbHelp(){
  	printf("\nBouton gauche pour tourner selon XY,\n");
  	printf("Bouton droit pour translater selon XY,\n");
  	printf("Bouton du milieu pour tourner et avancer selon Z.\n");
}

/** Gere les boutons de la souris */
void tbMouseFunc( int button, int state, int x, int y )
{
  /* enfoncer gauche */
  if( button==GLUT_LEFT_BUTTON && state==GLUT_DOWN ){
    tb_tournerXY = 1;
    tb_ancienX = x;
    tb_ancienY = y;
  }
  /* relacher gauche */
  else if( button==GLUT_LEFT_BUTTON && state==GLUT_UP ){
    tb_tournerXY = 0;
  }
  /* enfoncer milieu */
  if( button==GLUT_MIDDLE_BUTTON && state==GLUT_DOWN ){
    tb_bougerZ = 1;
    tb_ancienX = x;
    tb_ancienY = y;
  }
  /* relacher milieu */
  else if( button==GLUT_MIDDLE_BUTTON && state==GLUT_UP ){
    tb_bougerZ = 0;
  }
  /* enfoncer droit */
  else if( button==GLUT_RIGHT_BUTTON && state==GLUT_DOWN ){
    tb_translaterXY = 1;
    tb_ancienX = x;
    tb_ancienY = y;
  }
  /* relacher droit */
  else if( button==GLUT_RIGHT_BUTTON && state==GLUT_UP ){
    tb_translaterXY = 0;
  }
}


/** Traite le changement de position de la souris */
void tbMotionFunc( int x, int y )
{
	double dx,dy,nrm;

	if( tb_tournerXY || tb_translaterXY || tb_bougerZ )
	{
		/* deplacement */
		dx = x - tb_ancienX;
		dy = tb_ancienY - y; /* axe vertical dans l'autre sens */
		
		if( tb_tournerXY ){
			nrm = sqrt( dx*dx+dy*dy+dx*dx+dy*dy );
			glLoadIdentity();
			glRotatef( nrm, -dy, dx, 0 );/*axe perpendiculaire au deplacement*/
			glMultMatrixf( tb_rotation );
			glGetFloatv( GL_MODELVIEW_MATRIX, tb_rotation );
		}
		else if( tb_translaterXY ){
			tb_translation[0] += dx/100.0;
			tb_translation[1] += dy/100.0;
		}
		else {
			tb_translation[2] += dy/100.0;
			glLoadIdentity();
			glRotatef( dx, 0,0,-1 );/*axe perpendiculaire a l'ecran*/
			glMultMatrixf( tb_rotation );
			glGetFloatv( GL_MODELVIEW_MATRIX, tb_rotation );
		}
		tb_ancienX = x;
		tb_ancienY = y;
		glutPostRedisplay();
	}
}

#endif
